﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Scrabble
{
    class CrossCheck
    {
        public bool emptyRow;
        public bool emptyCol;
        private char[,] board;
        private HashSet<string> dictionary = new HashSet<string>();

        public CrossCheck(char[,] boardP, HashSet<string> dictionaryP)
        {
            board = boardP;
            dictionary = dictionaryP;
        }

        /*
         * Cross references words to make sure the the word we want to place will make valid words to any
         * words that we're adding letters to.
         *
         * If the Scrabble board looks like this:
         *       ?
         *       ?
         *       ? T O P
         *   C A T
         *   H   ?
         * D A M ?
         *   N   ?
         *   C   ?
         *   E   ?
         * 
         * codeWord = "   T     "
         * 
         * Then this function will be passed the following crossCheck array:
         * crossCheck(2):  TOP
         * crossCheck(5): DAM
         *
         * If passed the following words:
         * "  STEEL  " ->  true
         * "   TAP   " ->  true
         * "   TRUCK " ->  false  ("DAMU" is not a word)
         * "      IT " ->  false  (no connection to another word)
        */
        public bool crossCheck(string[] crossCheck, string codeWord, string word)
        {
            int letters = 0;
            bool oneLetter = false;
            char[] cw = codeWord.ToCharArray();
            char[] w = word.ToCharArray();

            //Check each cross reference
            for (int i=0; i<15; i++)
            {
                //If a letter in the word placement is already on the board, then keep track
                if (w[i] == cw[i] && w[i] != ' ')
                    letters++;

                //Empty checks
                if (w[i] == ' ') continue;
                if (crossCheck[i] == null) continue;
                oneLetter = true;

                //Checks to see if the word is in the dictionary    
                string crossRef = crossCheck[i];
                crossRef = Scrabble.Words.replace(crossRef, w[i], crossRef.IndexOf(' '));
                if (!dictionary.Contains(crossRef))
                    return false;
            }

            return (letters > 0 && letters < word.Trim().Length) || oneLetter;
        }

        /*
         * Creates a string[] of cross references that line up with a code word.
         *
         * For example, if the scrabble board were:
         *       ?      
         *       ? T O P
         *   C A T      
         *   H   ?      
         * D A M ? E D
         *   N   ?
         *   C   ?
         *   E   ?
         *
         * The code word is: "??T?????"
         * createCrossCheck(1) = Empty
         * createCrossCheck(2) = " TOP"
         * createCrossCheck(3) = Empty
         * createCrossCheck(4) = Empty
         * createCrossCheck(5) = "DAM ED"
         * createCrossCheck(6) = Empty
         * createCrossCheck(7) = Empty
         * createCrossCheck(8) = Empty
         */
        public string[] createCrossCheck(int r, int c)
        {
            if (r == -1)
                return CrossCheckColumn(c);
            else if (c == -1)
                return CrossCheckRow(r);
            else
                return new string[15];
        }


        //PRIVATE HELPER METHODS
        /*
         * Creates CrossChecks for a codeword that is verticle
        */
        private string[] CrossCheckColumn(int c)
        {
            string[] cross = new string[15];
            emptyCol = true;

            for (int i = 0; i < 15; i++)
            {
                //Check two conditions

                // 1) Cross reference must have a " " at this point
                if (board[i, c] != ' ') continue;

                // 2) Cross reference must have a letter to attach 
                //    to on one of it's sides
                bool cont = true;
                if (c < 14)
                    if (board[i, c + 1] != ' ')
                        cont = false;                
                if (c > 0)
                    if (board[i, c - 1] != ' ')
                        cont = false;
                if (cont) continue;

                string word = " ";
                int move;

                //Add right
                if (c < 14)
                    for (move = c + 1; move < 15; move++)
                    {
                        if (board[i, move] == ' ') break;
                        word += board[i, move];
                    }
                //Add left
                if (c > 0)
                    for (move = c - 1; move >= 0; move--)
                    {
                        if (board[i, move] == ' ') break;
                        word = String.Concat(board[i, move], word);
                    }

                cross[i] = word;
                emptyCol = false;
            }

            return cross;
        }

        /*
         * Creates CrossChecks for a codeword that is horizontal
        */
        private string[] CrossCheckRow(int r)
        {
            string[] cross = new string[15];
            emptyRow = true;

            for (int i = 0; i < 15; i++)
            {
                //Check two conditions

                // 1) Cross reference must have a " " at this point
                if (board[r, i] != ' ') continue;

                // 2) Cross reference must have a letter to attach 
                //    to on one of it's sides
                bool cont = true;
                if (r < 14)
                    if (board[r + 1, i] != ' ')
                        cont = false;
                if (r > 0)
                    if (board[r - 1, i] != ' ')
                        cont = false;
                if (cont) continue;

                string word = " ";
                int move;

                //Add down
                if (r < 14)
                    for (move = r + 1; move < 15; move++)
                    {
                        if (board[move, i] == ' ') break;
                        word += board[move, i];
                    }
                //Add up
                if (r > 0)
                    for (move = r - 1; move >= 0; move--)
                    {
                        if (board[move, i] == ' ') break;
                        word = String.Concat(board[move, i], word);
                    }

                cross[i] = word;
                emptyRow = false;
            }

            return cross;
        }
    }
}
